## ex2.py
# AE3M33UI - Artificial Intellignece course, FEE CTU in Prague
#
# Exercise 2: Linear regression
# The goal of this exercise is to 
#  * help the students better understand the linear regression model, and to
#  * make the familiar with the general process of model fitting.
#
# In this exercise, we will use the auto-mpg.csv dataset,
# which contains examples of cars and their various features like
# mpg, number of cylinders, displacement, horse power, weight, 
# acceleration, manufacturing year, origin (USA, EUR, JAP), and its name.
# 
# For the simple regression case, we shall study the relation of 
# horse powers and displacement, i.e. model hp=h(disp).

import numpy as np
from matplotlib.pyplot import *
from sklearn import linear_model

from linreg import *

## Load data for simple regression
X,y = load_mpg_for_simple_regression()
print('\n==Simple regression==')
print('The shapes of X and y are:')
print(X.shape)
print(y.shape)

## Plot the input data
figure()
scatter(X,y)
xlabel('disp', fontsize=16)
ylabel('hp', fontsize=16)
ax = axis()

## Estimate the model parameters by hand
# <PUT YOUR ESTIMATE OF w_0 AND w_1 HERE>
w_0 = 0   
w_1 = 1
wguess = np.array([w_0, w_1])
plot_predictions(wguess, c='g')

## Compute the error of the model
print('\nThe hand-crafted coefficients:')
print(wguess)
err = compute_cost_regr_lin(wguess, X, y)
print('The error:')
print(err)

## Find optimal weights by minimization of J
w1 = fit_regr_lin_by_minimization(X,y)
print('\nCoefficients found by minimization of J:')
print(w1)
err = compute_cost_regr_lin(w1, X, y)
print('The error:')
print(err)

## Find optimal weights by normal equation
w2 = fit_regr_lin_by_normal_equation(X,y)
print('\nCoefficients found by normal equation:')
print(w2)
err = compute_cost_regr_lin(w2, X, y)
print('The error:')
print(err)
plot_predictions(w2, c='r')
legend(['hand-crafted','optimal'],loc='upper left')

## Use scikit-learn package to train the model
# <ADD THE CODE WHICH CREATES AN INSTANCE OF LINEAR REGRESSION,
# AND TRAINS IT ON THE TRAINING DATA>
# lr = ...
# ...
w3 = np.array([lr.intercept_, lr.coef_[0]])
print('\nCoefficients found by sklearn.linear_model.LinearRegression:')
print(w3)
err = compute_cost_regr_lin(w3, X, y)
print('The error:')
print(err)

## Multivariate regression
X,y = load_mpg_for_multiple_regression()
print('\n==Multivariate regression==')
print('The shapes of X and y are:')
print(X.shape)
print(y.shape)

# <MODIFY THE FOLLOWING WITH YOUR CODE>
# # Fit the regression model by any method
# wmulti = ...
# print('\nCoefficients for multivariate regression model:')
# print(wmulti)
# # Compute the model error
# err = ...
# print('The error:')
# print(err)



