import numpy as np
import pandas as pd
import random
from scipy.optimize import minimize
from matplotlib.pyplot import *

def load_mpg_as_dataframe():
    """Load 'auto-mpg.csv' and return the disp and hp columns as numpy arrays."""
    # Using pandas to import the data; convenient, but a bit overkill
    df = pd.read_csv('auto-mpg.csv', 
        skipinitialspace=True, 
        names=['mpg', 'cyl', 'disp', 'hp', 'wgt', 'acc', 'year', 'orig', 'name'])
    return df

def load_mpg_for_simple_regression():
    """Load 'auto-mpg.csv' and return the disp and hp columns as numpy arrays."""
    df = load_mpg_as_dataframe()
    X = df.disp.values[:, np.newaxis]
    y = df.hp.values
    return X, y
    
def load_mpg_for_multiple_regression():
    """Load 'auto-mpg.csv' and return the disp and hp columns as numpy arrays."""
    df = load_mpg_as_dataframe()
    X = df.xs(['mpg','cyl','disp','wgt','acc'],axis=1).values
    y = df.hp.values
    return X, y
    
def plot_predictions(w, c='r'):
    """Plot predictions of the linear model with parameters w."""
    # Assuming there exists a plot of the data,
    # read the axes limits:
    ax = axis()
    # Build the test data as xmin and xmax:
    Xtst = np.array(ax[0:2])[:,np.newaxis]
    # Find the predictions
    ytst = pred_regr_lin(w,Xtst)
    # Plot them
    plot(Xtst,ytst, lw=3, c=c)
    # Set axes limits as they were before
    axis(ax)
    
def homogenize(X):
    """Return X with a prepended column of ones."""
    # MODIFY THE FOLLOWING WITH YOUR CODE
    raise NotImplementedError
    # hX = ...
    # return hX
    
def pred_regr_lin(w,X):
    """Predict the values of y for inputs X, given the linear model parameters w."""
    # MODIFY THE FOLLOWING WITH YOUR CODE    
    raise NotImplementedError
    # yhat = ...
    # return yhat
    
def compute_err_MSE(y, yhat):
    """Return the mean squared error given the true values of y and predictions yhat."""
    # MODIFY THE FOLLOWING WITH YOUR CODE    
    raise NotImplementedError
    # err = ...
    # return err
    
def compute_cost_regr_lin(w,X,y):
    """Return the cost of lin. reg. and its gradient at point w, given data X and y."""
    yhat = pred_regr_lin(w,X)
    err = compute_err_MSE(y, yhat)
    return err

def fit_regr_lin_by_minimization(X,y):
    """Return the parameters of linear model trained by MSE minimization."""
    # MODIFY THE FOLLOWING WITH YOUR CODE    
    raise NotImplementedError
    # Define f(w) somehow, see the exercise description doc.
    # w_init = np.array([0,0])
    # res = minimize( ... )
    # w = res.x
    # return w
    
def fit_regr_lin_by_normal_equation(X,y):
    """Return the parameters of linear model using normal equation."""
    # MODIFY THE FOLLOWING WITH YOUR CODE    
    raise NotImplementedError
    # w = ...
    # return w